/* SDL_Rect */
typedef struct SDL_Rect {
    Sint16 x, y;
    Uint16 w, h;
} SDL_Rect;

typedef struct SDL_Palette {
    int       ncolors;
    SDL_Color *colors;
} SDL_Palette;
/*@}*/

typedef struct SDL_PixelFormat {
    SDL_Palette *palette;
    Uint8  BitsPerPixel;
    Uint8  BytesPerPixel;
    Uint8  Rloss;
    Uint8  Gloss;
    Uint8  Bloss;
    Uint8  Aloss;
    Uint8  Rshift;
    Uint8  Gshift;
    Uint8  Bshift;
    Uint8  Ashift;
    Uint32 Rmask;
    Uint32 Gmask;
    Uint32 Bmask;
    Uint32 Amask;

    /** RGB color key information */
    Uint32 colorkey;
    /** Alpha value information (per-surface alpha) */
    Uint8  alpha;
} SDL_PixelFormat;

typedef struct SDL_Surface {
    Uint32 flags;/**< Read-only */
    SDL_PixelFormat *format;/**< Read-only */
    int w, h;/**< Read-only */
    Uint16 pitch;/**< Read-only */
    void *pixels;/**< Read-write */
    int offset;/**< Private */

    /** Hardware-specific surface info */
    struct private_hwdata *hwdata;

    /** clipping information */
    SDL_Rect clip_rect;/**< Read-only */
    Uint32 unused1;/**< for binary compatibility */

    /** Allow recursive locks */
    Uint32 locked;/**< Private */

    /** info for fast blit mapping to other surfaces */
    struct SDL_BlitMap *map;/**< Private */

    /** format version, bumped at every change to invalidate blit maps */
    unsigned int format_version;/**< Private */

    /** Reference count -- used when freeing surface */
    int refcount;/**< Read-mostly */
} SDL_Surface;

typedef struct SDL_Overlay {
    Uint32 format;/**< Read-only */
    int w, h;/**< Read-only */
    int planes;/**< Read-only */
    Uint16 *pitches;/**< Read-only */
    Uint8 **pixels;/**< Read-write */

    /** @name Hardware-specific surface info */
        /*@{*/
    struct private_yuvhwfuncs *hwfuncs;
    struct private_yuvhwdata *hwdata;
        /*@{*/

    /** @name Special flags */
        /*@{*/
    Uint32 hw_overlay :1;/**< Flag: This overlay hardware accelerated? */
    Uint32 UnusedBits :31;
        /*@}*/
} SDL_Overlay;

typedef struct SDL_VideoInfo {
    Uint32 hw_available :1;/**< Flag: Can you create hardware surfaces? */
    Uint32 wm_available :1;/**< Flag: Can you talk to a window manager? */
    Uint32 UnusedBits1  :6;
    Uint32 UnusedBits2  :1;
    Uint32 blit_hw      :1;/**< Flag: Accelerated blits HW --> HW */
    Uint32 blit_hw_CC   :1;/**< Flag: Accelerated blits with Colorkey */
    Uint32 blit_hw_A    :1;/**< Flag: Accelerated blits with Alpha */
    Uint32 blit_sw      :1;/**< Flag: Accelerated blits SW --> HW */
    Uint32 blit_sw_CC   :1;/**< Flag: Accelerated blits with Colorkey */
    Uint32 blit_sw_A    :1;/**< Flag: Accelerated blits with Alpha */
    Uint32 blit_fill    :1;/**< Flag: Accelerated color fill */
    Uint32 UnusedBits3  :16;
    Uint32 video_mem;/**< The total amount of video memory (in K) */
    SDL_PixelFormat *vfmt;/**< Value: The format of the video surface */
    int    current_w;/**< Value: The current video mode width */
    int    current_h;/**< Value: The current video mode height */
} SDL_VideoInfo;

#define SDL_SWSURFACE 0x00000000 /* Surface is in system memory */
#define SDL_HWSURFACE 0x00000001 /* Surface is in video memory */
#define SDL_ASYNCBLIT 0x00000004 /* Use asynchronous blits if possible */

/* Available for SDL_SetVideoMode() */
#define SDL_ANYFORMAT   0x10000000 /* Allow any video depth/pixel-format */
#define SDL_HWPALETTE   0x20000000 /* Surface has exclusive palette */
#define SDL_DOUBLEBUF   0x40000000 /* Set up double-buffered video mode */
#define SDL_FULLSCREEN  0x80000000 /* Surface is a full screen display */
#define SDL_OPENGL      0x00000002      /* Create an OpenGL rendering context */
#define SDL_OPENGLBLIT  0x0000000A /* Create an OpenGL rendering context and use it for blitting */
#define SDL_RESIZABLE   0x00000010 /* This video mode may be resized */
#define SDL_NOFRAME     0x00000020 /* No window caption or edge frame */

/* Used internally (read-only) */
#define SDL_HWACCEL     0x00000100 /* Blit uses hardware acceleration */
#define SDL_SRCCOLORKEY 0x00001000 /* Blit uses a source color key */
#define SDL_RLEACCELOK  0x00002000 /* Private flag */
#define SDL_RLEACCEL    0x00004000 /* Surface is RLE encoded */
#define SDL_SRCALPHA    0x00010000 /* Blit uses source alpha blending */
#define SDL_PREALLOC    0x01000000 /* Surface uses preallocated memory */

#define SDL_YV12_OVERLAY  0x32315659 /* Planar mode: Y + V + U  (3 planes) */
#define SDL_IYUV_OVERLAY  0x56555949 /* Planar mode: Y + U + V  (3 planes) */
#define SDL_YUY2_OVERLAY  0x32595559 /* Packed mode: Y0+U0+Y1+V0 (1 plane) */
#define SDL_UYVY_OVERLAY  0x59565955 /* Packed mode: U0+Y0+V0+Y1 (1 plane) */
#define SDL_YVYU_OVERLAY  0x55595659 /* Packed mode: Y0+V0+Y1+U0 (1 plane) */

/* flags for SDL_SetPalette() */
#define SDL_LOGPAL 0x01
#define SDL_PHYSPAL 0x02

/* Public enumeration for setting the OpenGL window attributes. */
typedef enum {
    SDL_GL_RED_SIZE,
    SDL_GL_GREEN_SIZE,
    SDL_GL_BLUE_SIZE,
    SDL_GL_ALPHA_SIZE,
    SDL_GL_BUFFER_SIZE,
    SDL_GL_DOUBLEBUFFER,
    SDL_GL_DEPTH_SIZE,
    SDL_GL_STENCIL_SIZE,
    SDL_GL_ACCUM_RED_SIZE,
    SDL_GL_ACCUM_GREEN_SIZE,
    SDL_GL_ACCUM_BLUE_SIZE,
    SDL_GL_ACCUM_ALPHA_SIZE,
    SDL_GL_STEREO,
    SDL_GL_MULTISAMPLEBUFFERS,
    SDL_GL_MULTISAMPLESAMPLES,
    SDL_GL_ACCELERATED_VISUAL,
    SDL_GL_SWAP_CONTROL
} SDL_GLattr;

typedef enum {
    SDL_GRAB_QUERY = -1,
    SDL_GRAB_OFF = 0,
    SDL_GRAB_ON = 1,
    SDL_GRAB_FULLSCREEN /* Used internally */
} SDL_GrabMode;

/* ----------------------------- */
/* Returns a pointer to the current display surface */
extern SDL_Surface * SDL_GetVideoSurface(void);
/* Returns a pointer to information about the video hardware */
extern const SDL_VideoInfo * SDL_GetVideoInfo(void);
/* Obtain the name of the video driver */
extern char * SDL_VideoDriverName(char *namebuf, int maxlen);
/* Returns a pointer to an array of available screen dimensions for the given format and video flags */
extern SDL_Rect ** SDL_ListModes(SDL_PixelFormat *format, Uint32 flags);
/* Checks to see if a particular video mode is supported. */
extern int SDL_VideoModeOK(int width, int height, int bpp, Uint32 flags);
/* Sets up a video mode with the specified width, height and bits-per-pixel. */
extern SDL_Surface * SDL_SetVideoMode (int width, int height, int bpp, Uint32 flags);
/* .. */
extern int SDL_VideoInit(const char *driver_name, Uint32 flags);
/* .. */
extern void SDL_VideoQuit(void);
/* Makes sure the given area is updated on the given screen. */
extern void SDL_UpdateRect (SDL_Surface *screen, Sint32 x, Sint32 y, Uint32 w, Uint32 h);
/* akes sure the given list of rectangles is updated on the given screen. */
extern void SDL_UpdateRects (SDL_Surface *screen, int numrects, SDL_Rect *rects);
/* Swaps screen buffers */
extern int SDL_Flip(SDL_Surface *screen);
/* Sets a portion of the colormap for the given 8-bit surface. */
extern int SDL_SetColors(SDL_Surface *surface, SDL_Color *colors, int firstcolor, int ncolors);
/* Sets the colors in the palette of an 8-bit surface. */
extern int SDL_SetPalette(SDL_Surface *surface, int flags, SDL_Color *colors, int firstcolor, int ncolors);
/* Sets the color gamma function for the display */
extern int SDL_SetGamma(float red, float green, float blue);
/* Gets the color gamma lookup tables for the display */
extern int SDL_GetGammaRamp(Uint16 *red, Uint16 *green, Uint16 *blue);
/* Sets the color gamma lookup tables for the display */
extern int SDL_SetGammaRamp(const Uint16 *red, const Uint16 *green, const Uint16 *blue);
/* Maps a RGB color value to a pixel format. */
extern Uint32 SDL_MapRGB (const SDL_PixelFormat * const format, const Uint8 r, const Uint8 g, const Uint8 b);
/* Maps a RGBA color value to a pixel format. */
extern Uint32 SDL_MapRGBA (const SDL_PixelFormat * const format, const Uint8 r, const Uint8 g, const Uint8 b, const Uint8 a);
/* Gets RGB values from a pixel in the specified pixel format. */
extern void SDL_GetRGB(Uint32 pixel, SDL_PixelFormat *fmt, Uint8 *r, Uint8 *g, Uint8 *b);
/* Gets RGBA values from a pixel in the specified pixel format. */
extern void SDL_GetRGBA(Uint32 pixel, SDL_PixelFormat *fmt, Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a);
/* Creates an empty SDL_Surface */
extern SDL_Surface * SDL_CreateRGBSurface (Uint32 flags, int width, int height, int depth, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
/* Creates an SDL_Surface from pixel data */
extern SDL_Surface * SDL_CreateRGBSurfaceFrom(void *pixels, int width, int height, int depth, int pitch, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
/* Frees (deletes) a SDL_Surface */
extern void SDL_FreeSurface(SDL_Surface *surface);
/* Locks a surface for direct access. */
extern int SDL_LockSurface(SDL_Surface *surface);
/* Unlocks a previously locked surface. */
extern void SDL_UnlockSurface(SDL_Surface *surface);
/* Converts a surface to the same format as another surface. */
extern SDL_Surface * SDL_ConvertSurface (SDL_Surface *src, SDL_PixelFormat *fmt, Uint32 flags);
/* Converts a surface to the display format */
extern SDL_Surface * SDL_DisplayFormat(SDL_Surface *surface);
/* Converts a surface to the display format */
extern SDL_Surface * SDL_DisplayFormatAlpha(SDL_Surface *surface);
/* Loads a Windows BMP file into an SDL_Surface. */
extern SDL_Surface * SDL_LoadBMP_RW(SDL_RWops *src, int freesrc);
/* Saves an SDL_Surface as a Windows BMP file. */
extern int SDL_SaveBMP(SDL_Surface *surface, const char *file);
/* ... */
extern int SDL_SaveBMP_RW (SDL_Surface *surface, SDL_RWops *dst, int freedst);
/* Sets the color key (transparent pixel) in a blittable surface and RLE acceleration. */
extern int SDL_SetColorKey (SDL_Surface *surface, Uint32 flag, Uint32 key);
/* Adjusts the alpha properties of a surface */
extern int SDL_SetAlpha(SDL_Surface *surface, Uint32 flag, Uint8 alpha);
/* Sets the clipping rectangle for a surface. */
extern SDL_bool SDL_SetClipRect(SDL_Surface *surface, const SDL_Rect *rect);
/* Gets the clipping rectangle for a surface. */
extern void SDL_GetClipRect(SDL_Surface *surface, SDL_Rect *rect);
/*  This function performs a fast blit from the source surface to the destination surface. */
extern int SDL_UpperBlit (SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect);
/*  This function performs a fast blit from the source surface to the destination surface. */
extern int SDL_LowerBlit (SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect);
/* This function performs a fast fill of the given rectangle with some color */
extern int SDL_FillRect (SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color);

extern SDL_Overlay * SDL_CreateYUVOverlay(int width, int height, Uint32 format, SDL_Surface *display);
extern int SDL_LockYUVOverlay(SDL_Overlay *overlay);
extern void SDL_UnlockYUVOverlay(SDL_Overlay *overlay);
extern int SDL_DisplayYUVOverlay(SDL_Overlay *overlay, SDL_Rect *dstrect);
extern void SDL_FreeYUVOverlay(SDL_Overlay *overlay);

extern int SDL_GL_LoadLibrary(const char *path);
extern void * SDL_GL_GetProcAddress(const char* proc);
extern int SDL_GL_SetAttribute(SDL_GLattr attr, int value);
extern int SDL_GL_GetAttribute(SDL_GLattr attr, int* value);
extern void SDL_GL_SwapBuffers(void);
extern void SDL_GL_UpdateRects(int numrects, SDL_Rect* rects);
extern void SDL_GL_Lock(void);
extern void SDL_GL_Unlock(void);
